import { CacheProvider, css, Global } from '@emotion/react';
import CssBaseline from '@mui/material/CssBaseline';
import { ThemeProvider } from '@mui/material/styles';
import { FC, PropsWithChildren, useMemo } from 'react';

import nuskinTheme from '../themes/genome-foundation';
import { generateGlobalColorCss, generateTextAlignCss, generateFontFamilyCssClass } from '../utils';
import { defaultCacheKey, getCacheKey } from './getCacheKey';
import { FoundationTheme } from '../themes';
import { NextGenThemeProviderProps } from './types';
import { useAppTheme } from './useAppTheme';

type PropType = FC<PropsWithChildren<NextGenThemeProviderProps>>;

export const importThemeFonts = (typography: FoundationTheme.Theme['fonts'], locale?: string) => {
    const localeFont = typography?.[locale as string || 'en-US'];
    const fonts = localeFont?.fontsToLoad || [];
     
    const googleFonts = fonts.filter((font) => font.type === 'GoogleFont');
    let googleFontFamily = googleFonts?.reduce((allFontImports, font, index) => {
        const hasItalics = Array.isArray(font.fontStyle?.[0]);
        const fontVariants = hasItalics ? ':ital,wght' : ':wght';

        let fontWeights = '';
        if (hasItalics) {
            fontWeights = font.fontStyle
                .map((fontStyle) => {
                    const [weight, italic] = fontStyle as [number, number];
                    let regularFontWeight = `0,${weight}`;
                    if (italic) {
                        regularFontWeight += `;1,${weight}`;
                    }
                    return regularFontWeight;
                })
                .join(';');
        } else {
            fontWeights = font.fontStyle.join(';');
        }

        const fontString = `${index > 0 ? 'family=' : ''}${font.fontName}${fontVariants}@${fontWeights}&`;
        allFontImports += fontString;

        return allFontImports;
    }, '');

    /*
     * [Todo] - Should handle for Custom fonts also
     */
    if (googleFontFamily) {
        googleFontFamily = `@import url('https://fonts.googleapis.com/css2?family=${googleFontFamily}display=swap');`;
    }
    return googleFontFamily;
};

export const NextGenThemeProvider: PropType = (props) => {
    const {
        isSSR = false,
        children, cacheKey = defaultCacheKey, theme = nuskinTheme, mode, locale } = props;

    const { appMuiTheme, isDarkMode } = useAppTheme({ mode, theme, locale });

    const cache = useMemo(() => getCacheKey(cacheKey), [cacheKey]);

    const GlobalCss = useMemo(() => {
        const fontSections = theme?.fonts?.[locale as string || 'en-US']?.section;
        return (
            <>
                <CssBaseline />
                <style jsx global>{`
                    ${importThemeFonts(theme.fonts, locale)}
                `}</style>
                <Global
                    styles={css`
                        ${generateFontFamilyCssClass(fontSections)}
                        ${generateTextAlignCss()}
                        ${generateGlobalColorCss(theme)}
                    `}
                />
            </>
        );
    }, [locale, theme]);

    const Content = (
        <ThemeProvider theme={appMuiTheme}>
            {GlobalCss}
            <main className={`theme-wrapper ${isDarkMode ? 'dark' : 'light'}`}>{children}</main>
        </ThemeProvider>
    );

    if (isSSR) {
        return <CacheProvider value={cache}>{Content}</CacheProvider>;
    }
    return Content;
};

export default NextGenThemeProvider;
